/*
 *  Copyright (C) 2005 SAN People
 *  Copyright (C) 2008 Atmel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/types.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/spi/spi.h>
#include <linux/spi/at73c213.h>
#include <linux/clk.h>

#include <mach/hardware.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/irq.h>
/*dm9000*/
#include <linux/dm9000.h>
#include <mach/at91sam9260_matrix.h>


#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/board.h>
#include <mach/gpio.h>
#include <mach/at91sam9_smc.h>
#include <linux/serial_8250.h>

#include "sam9_smc.h"
#include "generic.h"


static struct resource dm9000_matrix516_resource[] = {
	[0] = {
		.start	= AT91_CHIPSELECT_6,
		.end	= AT91_CHIPSELECT_6 + 3,
		.flags	= IORESOURCE_MEM
	},
	[1] = {
		.start	= AT91_CHIPSELECT_6 + 0x44,
		.end	= AT91_CHIPSELECT_6 + 0xFF,
		.flags	= IORESOURCE_MEM
	},
	[2] = {
		.start	= AT91_PIN_PC12,
		.end	= AT91_PIN_PC12,
		.flags	= IORESOURCE_IRQ  | IRQF_SHARED ,//| IORESOURCE_IRQ_HIGHEDGE,
	}
};

/* for the moment we limit ourselves to 16bit IO until some
 * better IO routines can be written and tested
*/

static struct dm9000_plat_data bast_matrix516_platdata = {
	.flags		= DM9000_PLATF_8BITONLY,
};

static struct platform_device dm9000_device = {
	.name		= "dm9000",
	.id		= 1,
	.num_resources	= ARRAY_SIZE(dm9000_matrix516_resource),
	.resource	= dm9000_matrix516_resource,
	.dev		= {
		.platform_data = &bast_matrix516_platdata,
	}
};

/*
 * SMC timings for the DM9000.
 * Note: These timings were calculated for MASTER_CLOCK = 100000000 according to the DM9000 timings.
 */
static struct sam9_smc_config __initdata dm9000_smc_config = {
	.ncs_read_setup		= 2*2,
	.nrd_setup			= 4*2,
	.ncs_write_setup	= 2*2,
	.nwe_setup			= 4*2,

	.ncs_read_pulse		= 8*2,
	.nrd_pulse			= 4*2,
	.ncs_write_pulse	= 8*2,
	.nwe_pulse			= 4*2,

	.read_cycle			= 16*2,
	.write_cycle		= 16*2,

	.mode				= AT91_SMC_READMODE | AT91_SMC_WRITEMODE | AT91_SMC_EXNWMODE_DISABLE | AT91_SMC_DBW_8,
	.tdf_cycles			= 3,
};

static void __init matrix516_add_device_dm9000(void)
{

	at91_set_B_periph(AT91_PIN_PC13,1);
	/* Configure chip-select 6 (DM9000) */
	sam9_smc_configure(6, &dm9000_smc_config);

	/* Configure Interrupt pin as input, no pull-up */
	at91_set_gpio_input(AT91_PIN_PC12, 1);

	platform_device_register(&dm9000_device);
}

static void __init matrix516_map_io(void)
{
	
	char *args = boot_command_line , *p;
	int console_artila = 0;
	
	/* Initialize processor: 18.432 MHz crystal */
	at91sam9260_initialize(18432000);
	
	//switch console for Artila
	p = strstr(args, "console=NULL");
	if (p != NULL) {console_artila = -1;}
	p = strstr(args, "console=ttyS1");
	if (p != NULL) {console_artila = 1;}
	p = strstr(args, "console=ttyS2");
	if (p != NULL) {console_artila = 2;}
	p = strstr(args, "console=ttyS3");
	if (p != NULL) {console_artila = 3;}
	p = strstr(args, "console=ttyS4");
	if (p != NULL) {console_artila = 4;}
	
	/* DGBU on ttyS0. (Rx & Tx only) */
	at91_register_uart(0, 0, 0);

	/* USART0 on ttyS1. (Rx, Tx, RTS, CTS) */
	at91_register_uart(AT91SAM9260_ID_US0, 2, ATMEL_UART_CTS | ATMEL_UART_RTS 
			   | ATMEL_UART_DTR | ATMEL_UART_DSR | ATMEL_UART_DCD);

	/* USART1 on ttyS2. (Rx, Tx, CTS, RTS, DTR, DSR, DCD, RI) */
	at91_register_uart(AT91SAM9260_ID_US1, 1, ATMEL_UART_CTS | ATMEL_UART_RTS);
	/* USART2 on ttyS3. (Rx, Tx, RTS, CTS) */
	at91_register_uart(AT91SAM9260_ID_US2, 3, ATMEL_UART_CTS | ATMEL_UART_RTS);
	/* USART3 on ttyS4. (Rx, Tx, RTS, CTS) */
	at91_register_uart(AT91SAM9260_ID_US3, 4, ATMEL_UART_CTS | ATMEL_UART_RTS);
	/* set serial console to ttyS0 (ie, DBGU) */
	if(console_artila != -1)
		at91_set_serial_console(console_artila);
}

static void __init matrix516_init_irq(void)
{
	at91sam9260_init_interrupts(NULL);
}


/*
 * USB Host port
 */
static struct at91_usbh_data __initdata matrix516_usbh_data = {
	.ports		= 2,
};

/*
 * USB Device port
 */
static struct at91_udc_data __initdata matrix516_udc_data = {
	.vbus_pin	= AT91_PIN_PC5,
	.pullup_pin	= 0,		/* pull-up driven by UDC */
};


/*
 * SPI devices.!defined(CONFIG_MMC_AT91)
 */
static struct spi_board_info matrix516_spi_devices[] = {
	{	/* DataFlash chip */
		.modalias	= "mtd_dataflash",
		.chip_select	= 1,
		.max_speed_hz	= 15 * 1000 * 1000,
		.bus_num	= 0,
	},{	/* DataFlash chip */
		.modalias	= "spidev",
		.chip_select	= 0,
		.max_speed_hz	= 15 * 1000 * 1000,
		.bus_num	= 1,
	},{	/* DataFlash chip */
		.modalias	= "spidev",
		.chip_select	= 3,
		.max_speed_hz	= 15 * 1000 * 1000,
		.bus_num	= 1,
	}
};


/*
 * MACB Ethernet device
 */
static struct at91_eth_data __initdata matrix516_macb_data = {
	.phy_irq_pin	= AT91_PIN_PC7,
	.is_rmii	= 1,
};

/* det_pin is not connected */
static struct atmel_nand_data __initdata matrix516_nand_data = {
	.ale		= 21,
	.cle		= 22,
	.rdy_pin	= AT91_PIN_PC0,
	.enable_pin	= AT91_PIN_PC14,
	//.partition_info	= nand_partitions,
#if defined(CONFIG_MTD_NAND_ATMEL_BUSWIDTH_16)
	.bus_width_16	= 1,
#else
	.bus_width_16	= 0,
#endif
};

static struct sam9_smc_config __initdata matrix516_nand_smc_config = {
	.ncs_read_setup		= 0,
	.nrd_setup		= 2,
	.ncs_write_setup	= 0,
	.nwe_setup		= 2,

	.ncs_read_pulse		= 4,
	.nrd_pulse		= 4,
	.ncs_write_pulse	= 4,
	.nwe_pulse		= 4,

	.read_cycle		= 7,
	.write_cycle		= 7,

	.mode			= AT91_SMC_READMODE | AT91_SMC_WRITEMODE | AT91_SMC_EXNWMODE_DISABLE,
	.tdf_cycles		= 3,
};

static void __init matrix516_add_device_nand(void)
{
	/* setup bus-width (8 or 16) */
	if (matrix516_nand_data.bus_width_16)
		matrix516_nand_smc_config.mode |= AT91_SMC_DBW_16;
	else
		matrix516_nand_smc_config.mode |= AT91_SMC_DBW_8;

	/* configure chip-select 3 (NAND) */
	sam9_smc_configure(3, &matrix516_nand_smc_config);

	at91_add_device_nand(&matrix516_nand_data);
}


/*
 * MCI (SD/MMC)
 * det_pin, wp_pin and vcc_pin are not connected
 */
static struct at91_mmc_data __initdata matrix516_mmc_data = {
	.slot_b		= 0,
	.wire4		= 1,
	.det_pin	= AT91_PIN_PC4,
};


/*
 * LEDs
 */
static struct gpio_led matrix516_leds[] = {
	{	
		.name			= "led0",
		.gpio			= AT91_PIN_PA22,
#ifdef CONFIG_MINI_MATRIX
		.active_low		= 0,
#else
		.active_low		= 1,
#endif
		.default_trigger	= "none",
	},
	{	
		.name			= "led1",
		.gpio			= AT91_PIN_PA25,
		.active_low		= 1,
		.default_trigger	= "none",
	},
	{	
		.name			= "led2",
		.gpio			= AT91_PIN_PA27,
		.active_low		= 1,
		.default_trigger	= "none",
	},
	/*{	
		.name			= "led3",
		.gpio			= AT91_PIN_PA28,
		.active_low		= 1,
		.default_trigger	= "none",
	},*/
};


static struct i2c_board_info __initdata matrix516_i2c_devices[] = {
	{
		I2C_BOARD_INFO("m41t81", 0x68),
	},
	{
		I2C_BOARD_INFO("pca9539", 0x76),
	},
	{
		I2C_BOARD_INFO("pca9539", 0x74),
	},
	{
		I2C_BOARD_INFO("tlv320aic23", 0x1B),
	},
	//{
	//	I2C_BOARD_INFO("tlv320aic23", 0x1A),
	//},
};

#if 1
/*---------------------------------------------------------------------
 * OX16C954 UART
 */
#define SERIAL_FLAGS	(UPF_BOOT_AUTOCONF | UPF_IOREMAP | UPF_SHARE_IRQ)
////#define SERIAL_CLK	(18432000)
//#define SERIAL_CLK	(1)//#define SERIAL_CLK	(7372800)
#define SERIAL_CLK	(14745600)
#define PORT(_base, _irq)				\
	{						\
		.mapbase	= _base,		\
		.irq		= _irq,			\
		.uartclk	= SERIAL_CLK,		\
		.iotype		= UPIO_MEM,		\
		.regshift	= 0,			\
		.flags		= SERIAL_FLAGS,		\
	}

static struct plat_serial8250_port matrix516_data[] = {
#if 1
	PORT(AT91_CHIPSELECT_5+0xd0, AT91_PIN_PC15),
	PORT(AT91_CHIPSELECT_5+0xb0, AT91_PIN_PC15),
	PORT(AT91_CHIPSELECT_5+0x70, AT91_PIN_PC15),
	PORT(AT91_CHIPSELECT_5+0xe0, AT91_PIN_PC15),
#else
	PORT(AT91_CHIPSELECT_5+0xd0, AT91SAM9260_ID_IRQ1),
	PORT(AT91_CHIPSELECT_5+0xb0, AT91SAM9260_ID_IRQ1),
	PORT(AT91_CHIPSELECT_5+0x70, AT91SAM9260_ID_IRQ1),
	PORT(AT91_CHIPSELECT_5+0xe0, AT91SAM9260_ID_IRQ1),
#endif
	{ },
};

static struct sam9_smc_config __initdata serial8250_smc_config = {
	.ncs_read_setup	 = 2*2,
	.nrd_setup		 = 6*2,
	.ncs_write_setup = 2*2,
	.nwe_setup		 = 6*2,
	
	.ncs_read_pulse  = 8*2,
	.nrd_pulse		 = 4*2,
	.ncs_write_pulse = 8*2,
	.nwe_pulse		 = 4*2,
	
	.read_cycle		 = 64,
	.write_cycle	 = 64,

	.mode			= AT91_SMC_READMODE | AT91_SMC_WRITEMODE | AT91_SMC_EXNWMODE_DISABLE  | AT91_SMC_DBW_8,
	.tdf_cycles		= 3,//12,//0,//1,//3,
};


static struct platform_device matrix516_device = {
	.name			= "serial8250",
	.id			= 5,//PLAT8250_DEV_PLATFORM,
	.dev			=
		{
			.platform_data	= &matrix516_data,
		},
	//.resource	= serial_resources,
	//.num_resources	= ARRAY_SIZE(serial_resources),
};

static void serial8250_setup_uart(void)
{
	unsigned long csa;
	
	//at91_set_B_periph(AT91_PIN_PC15,0);
	//
	at91_set_A_periph(AT91_PIN_PC9,1);
	
	csa = at91_sys_read(AT91_MATRIX_EBICSA);
	at91_sys_write(AT91_MATRIX_EBICSA, csa | AT91_MATRIX_CS5A);
	
	sam9_smc_configure(5, &serial8250_smc_config);
	
	at91_set_gpio_output(AT91_PIN_PB25, 1);
	at91_set_gpio_output(AT91_PIN_PB30, 1);
	at91_set_gpio_output(AT91_PIN_PB31, 1);
	at91_set_gpio_output(AT91_PIN_PC0, 1);
	at91_set_gpio_input(AT91_PIN_PC15, 0);
	at91_set_deglitch(AT91_PIN_PC15, 1);
	
	platform_device_register(&matrix516_device);
}
#endif
static void __init matrix516_board_init(void)
{
	/* Serial */
	//at91_add_device_serial();
	serial8250_setup_uart();
	//
	/* USB Host */
	at91_add_device_usbh(&matrix516_usbh_data);
	/* USB Device */
	at91_add_device_udc(&matrix516_udc_data);
	/* SPI */
	//Artila Ace set 
	at91_add_device_spi(matrix516_spi_devices, ARRAY_SIZE(matrix516_spi_devices));//0);//
	/* NAND */
	matrix516_add_device_nand();
	/* Ethernet */
	at91_add_device_eth(&matrix516_macb_data);
	/* MMC */
	at91_add_device_mmc(0, &matrix516_mmc_data);
	/* I2C */
	at91_add_device_i2c(matrix516_i2c_devices, ARRAY_SIZE(matrix516_i2c_devices));
	/* LEDs */
	at91_gpio_leds(matrix516_leds, 4);//ARRAY_SIZE(matrix516_leds));
	/*dm9000*/
	matrix516_add_device_dm9000();
	//at73c213_set_clk(&at73c213_data);
	at91_add_device_ssc(AT91SAM9260_ID_SSC, ATMEL_SSC_TX);
	
	
	
	at91_add_device_serial();
}

MACHINE_START(MATRIX516, "Matrix516")
	/* Maintainer: Atmel */
	.phys_io	= AT91_BASE_SYS,
	.io_pg_offst	= (AT91_VA_BASE_SYS >> 18) & 0xfffc,
	.boot_params	= AT91_SDRAM_BASE + 0x100,
	.timer		= &at91sam926x_timer,
	.map_io		= matrix516_map_io,
	.init_irq	= matrix516_init_irq,
	.init_machine	= matrix516_board_init,
MACHINE_END
