/*
 *  Copyright (C) 2005 SAN People
 *  Copyright (C) 2008 Atmel
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/types.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/module.h>
#include <linux/platform_device.h>
#include <linux/spi/spi.h>
#include <linux/spi/at73c213.h>
#include <linux/clk.h>
/*dm9000*/
#include <linux/dm9000.h>
#include <mach/at91sam9260_matrix.h>
#include <linux/serial_8250.h>
#include <mach/hardware.h>
#include <asm/setup.h>
#include <asm/mach-types.h>
#include <asm/irq.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/irq.h>

#include <mach/board.h>
#include <mach/gpio.h>
#include <mach/at91sam9_smc.h>

#include <linux/can/platform/sja1000.h>

#include "sam9_smc.h"
#include "generic.h"

static struct resource dm9000_matrix518_resource[] = {
	[0] = {
		.start	= AT91_CHIPSELECT_6,
		.end	= AT91_CHIPSELECT_6 + 3,
		.flags	= IORESOURCE_MEM
	},
	[1] = {
		.start	= AT91_CHIPSELECT_6 + 0x44,
		.end	= AT91_CHIPSELECT_6 + 0xFF,
		.flags	= IORESOURCE_MEM
	},
	[2] = {
		.start	= AT91_PIN_PC12,
		.end	= AT91_PIN_PC12,
		.flags	= IORESOURCE_IRQ  | IRQF_SHARED//| IORESOURCE_IRQ_HIGHEDGE,
	}
};

/* for the moment we limit ourselves to 16bit IO until some
 * better IO routines can be written and tested
*/

static struct dm9000_plat_data bast_matrix518_platdata = {
	.flags		= DM9000_PLATF_8BITONLY,
};

static struct platform_device dm9000_device = {
	.name		= "dm9000",
	.id		= 1,
	.num_resources	= ARRAY_SIZE(dm9000_matrix518_resource),
	.resource	= dm9000_matrix518_resource,
	.dev		= {
		.platform_data = &bast_matrix518_platdata,
	}
};

/*
 * SMC timings for the DM9000.
 * Note: These timings were calculated for MASTER_CLOCK = 100000000 according to the DM9000 timings.
 */
static struct sam9_smc_config __initdata dm9000_smc_config = {
	.ncs_read_setup		= 2,
	.nrd_setup		= 4,
	.ncs_write_setup	= 2,
	.nwe_setup		= 4,

	.ncs_read_pulse		= 8,
	.nrd_pulse		= 4,
	.ncs_write_pulse	= 8,
	.nwe_pulse		= 4,

	.read_cycle		= 16,
	.write_cycle		= 16,

	.mode			= AT91_SMC_READMODE | AT91_SMC_WRITEMODE | AT91_SMC_EXNWMODE_DISABLE | AT91_SMC_DBW_8,
	.tdf_cycles		= 3,
};

static void __init matrix518_add_device_dm9000(void)
{

	at91_set_B_periph(AT91_PIN_PC13,1);
	/* Configure chip-select 6 (DM9000) */
	sam9_smc_configure(6, &dm9000_smc_config);

	/* Configure Interrupt pin as input, no pull-up */
	at91_set_gpio_input(AT91_PIN_PC12, 0);
	//at91_set_A_periph(AT91_PIN_PC12,0);

	platform_device_register(&dm9000_device);
}

static void __init m502_map_io(void)
{
	
	char *args = boot_command_line , *p;
	int console_artila = 0;
	
	/* Initialize processor: 18.432 MHz crystal */
	at91sam9260_initialize(18432000);
	
	//switch console for Artila
	p = strstr(args, "console=NULL");
	if (p != NULL) {console_artila = -1;}
	p = strstr(args, "console=ttyS1");
	if (p != NULL) {console_artila = 1;}
	p = strstr(args, "console=ttyS2");
	if (p != NULL) {console_artila = 2;}
	p = strstr(args, "console=ttyS3");
	if (p != NULL) {console_artila = 3;}
	p = strstr(args, "console=ttyS4");
	if (p != NULL) {console_artila = 4;}
	
	/* DGBU on ttyS0. (Rx & Tx only) */
	at91_register_uart(0, 0, 0);

	/* USART0 on ttyS1. (Rx, Tx, RTS, CTS) */
	at91_register_uart(AT91SAM9260_ID_US0, 2, ATMEL_UART_CTS | ATMEL_UART_RTS 
			   | ATMEL_UART_DTR | ATMEL_UART_DSR | ATMEL_UART_DCD);

	/* USART1 on ttyS2. (Rx, Tx, CTS, RTS, DTR, DSR, DCD, RI) */
	at91_register_uart(AT91SAM9260_ID_US1, 1, ATMEL_UART_CTS | ATMEL_UART_RTS);
	
	/* USART2 on ttyS2. (Rx, Tx, CTS, RTS, DTR, DSR, DCD, RI) */
	at91_register_uart(AT91SAM9260_ID_US2, 3, ATMEL_UART_CTS | ATMEL_UART_RTS);
	
	/* USART3 on ttyS2. (Rx, Tx, CTS, RTS, DTR, DSR, DCD, RI) */
	at91_register_uart(AT91SAM9260_ID_US3, 4, ATMEL_UART_CTS | ATMEL_UART_RTS);

	if(console_artila != -1)
		at91_set_serial_console(console_artila);
}

static void __init m502_init_irq(void)
{
	at91sam9260_init_interrupts(NULL);
}


/*
 * USB Host port
 */
static struct at91_usbh_data __initdata m502_usbh_data = {
	.ports		= 2,
};

/*
 * USB Device port
 */
static struct at91_udc_data __initdata m502_udc_data = {
	.vbus_pin	= AT91_PIN_PC5,
	.pullup_pin	= 0,		/* pull-up driven by UDC */
};


/*
 * SPI devices.!defined(CONFIG_MMC_AT91)
 */
static struct spi_board_info m502_spi_devices[] = {
	{	/* DataFlash chip */
		.modalias	= "mtd_dataflash",
		.chip_select	= 1,
		.max_speed_hz	= 15 * 1000 * 1000,
		.bus_num	= 0,
	}
	
	#if 0
	,{	/* DataFlash chip */
		.modalias	= "spidev",
		.chip_select	= 0,
		.max_speed_hz	= 15 * 1000 * 1000,
		.bus_num	= 1,
	},{	/* DataFlash chip */
		.modalias	= "spidev",
		.chip_select	= 3,
		.max_speed_hz	= 15 * 1000 * 1000,
		.bus_num	= 1,
	}
	#endif
	
};


/*
 * MACB Ethernet device
 */
static struct at91_eth_data __initdata m502_macb_data = {
	.phy_irq_pin	= AT91_PIN_PC7,
	.is_rmii	= 1,
};


/* det_pin is not connected */
static struct atmel_nand_data __initdata m502_nand_data = {
	.ale		= 21,
	.cle		= 22,
	.rdy_pin	= AT91_PIN_PC0,
	.enable_pin	= AT91_PIN_PC14,
	//.partition_info	= nand_partitions,
#if defined(CONFIG_MTD_NAND_ATMEL_BUSWIDTH_16)
	.bus_width_16	= 1,
#else
	.bus_width_16	= 0,
#endif
};

static struct sam9_smc_config __initdata m502_nand_smc_config = {
	.ncs_read_setup		= 0,
	.nrd_setup		= 2,
	.ncs_write_setup	= 0,
	.nwe_setup		= 2,

	.ncs_read_pulse		= 4,
	.nrd_pulse		= 4,
	.ncs_write_pulse	= 4,
	.nwe_pulse		= 4,

	.read_cycle		= 7,
	.write_cycle		= 7,

	.mode			= AT91_SMC_READMODE | AT91_SMC_WRITEMODE | AT91_SMC_EXNWMODE_DISABLE,
	.tdf_cycles		= 3,
};

static void __init m502_add_device_nand(void)
{
	/* setup bus-width (8 or 16) */
	if (m502_nand_data.bus_width_16)
		m502_nand_smc_config.mode |= AT91_SMC_DBW_16;
	else
		m502_nand_smc_config.mode |= AT91_SMC_DBW_8;

	/* configure chip-select 3 (NAND) */
	sam9_smc_configure(3, &m502_nand_smc_config);

	at91_add_device_nand(&m502_nand_data);
}

#if 1
/*
 * MCI (SD/MMC)
 * det_pin, wp_pin and vcc_pin are not connected
 */
static struct at91_mmc_data __initdata m502_mmc_data = {
	.slot_b		= 0,
	.wire4		= 1,
	.det_pin	= AT91_PIN_PC4,
};
#endif

/*
 * LEDs
 */
static struct gpio_led m502_leds[] = {
	{	
		.name			= "led0",
		.gpio			= AT91_PIN_PA22,
#ifdef CONFIG_MINI_MATRIX
		.active_low		= 0,
#else
		.active_low		= 1,
#endif
		.default_trigger	= "none",
	},
	{	
		.name			= "led1",
		.gpio			= AT91_PIN_PA25,
		.active_low		= 1,
		.default_trigger	= "none",
	},
	{	
		.name			= "led2",
		.gpio			= AT91_PIN_PA27,
		.active_low		= 1,
		.default_trigger	= "none",
	},
	{	
		.name			= "led3",
		.gpio			= AT91_PIN_PA28,
		.active_low		= 1,
		.default_trigger	= "none",
	},

};


void pac4010_id_init(void) {
	at91_set_gpio_input(AT91_PIN_PB20, 1);
	at91_set_gpio_input(AT91_PIN_PB21, 1);
	at91_set_gpio_input(AT91_PIN_PC1, 1);
}

static struct i2c_board_info __initdata m502_i2c_devices[] = {
	{
		I2C_BOARD_INFO("m41t81", 0x68),
	},
	{
		I2C_BOARD_INFO("pca9539", 0x76),
		//I2C_BOARD_INFO("pca953x", 0x76),
	},
	{
		I2C_BOARD_INFO("pca9539", 0x74),
	},
};

static void __init m502_board_init(void)
{
	/*ID pin*/
	pac4010_id_init();
	/* I2C */
	at91_add_device_i2c(m502_i2c_devices, ARRAY_SIZE(m502_i2c_devices));
	/* Serial */
	at91_add_device_serial();
	/* USB Host */
	at91_add_device_usbh(&m502_usbh_data);
	/* USB Device */
	at91_add_device_udc(&m502_udc_data);
	/* SPI */
	//Artila Ace set 
	at91_add_device_spi(m502_spi_devices, ARRAY_SIZE(m502_spi_devices));//0);//
	/* NAND */
	m502_add_device_nand();
	/* Ethernet */
	at91_add_device_eth(&m502_macb_data);
	/* DM9000 */
	matrix518_add_device_dm9000();
	/* MMC */
	at91_add_device_mmc(0, &m502_mmc_data);
	/* LEDs */
	at91_gpio_leds(m502_leds, 4);//ARRAY_SIZE(m502_leds));
	
}

MACHINE_START(PAC4010, "PAC-4000")
	/* Maintainer: Atmel */
	.phys_io	= AT91_BASE_SYS,
	.io_pg_offst	= (AT91_VA_BASE_SYS >> 18) & 0xfffc,
	.boot_params	= AT91_SDRAM_BASE + 0x100,
	.timer		= &at91sam926x_timer,
	.map_io		= m502_map_io,
	.init_irq	= m502_init_irq,
	.init_machine	= m502_board_init,
MACHINE_END
