/*
 * LCD interface for M-502
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1996, 1997 by Andrew Bose
 *
 * Linux kernel version history:
 *       March 2001: Ported from 2.0.34  by Liam Davies
 *
 */



#include <linux/types.h>
#include <linux/errno.h>
#include <linux/miscdevice.h>
#include <linux/slab.h>
#include <linux/ioport.h>
#include <linux/fcntl.h>
#include <linux/netdevice.h>
#include <linux/sched.h>
#include <linux/delay.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/system.h>
#include <linux/delay.h>

#include <mach/gpio.h>
#include <mach/at91sam9_smc.h>
#include <mach/board.h>
#include <mach/at91sam9260_matrix.h>
#include <asm/mach/arch.h>
#include <asm/pca9539.h>

#include "lcd_wg.h"

static DEFINE_SPINLOCK(lcd_lock);

static int lcd_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long arg);

static unsigned int lcd_present = 1;

/* used in arch/mips/cobalt/reset.c */
int led_state = 0;

static int lcd_base;
static u8 lcd_read_status(void);
#define MISC_LCD_MINOR 240

#define BusyCheck()	
//while ((lcd_read_status() & 0x80) == 0x80)

#if defined(CONFIG_TULIP) && 0

#define MAX_INTERFACES	8
static linkcheck_func_t linkcheck_callbacks[MAX_INTERFACES];
static void *linkcheck_cookies[MAX_INTERFACES];


int lcd_register_linkcheck_func(int iface_num, void *func, void *cookie)
{
	if (iface_num < 0 ||
	    iface_num >= MAX_INTERFACES ||
	    linkcheck_callbacks[iface_num] != NULL)
		return -1;
	linkcheck_callbacks[iface_num] = (linkcheck_func_t) func;
	linkcheck_cookies[iface_num] = cookie;
	return 0;
}
#endif

#define at91_lcm_read(reg)		readb(lcd_base + (reg))//__raw_readb(lcd_base + (reg))
#define at91_lcm_write(reg, val)	writeb((val), lcd_base + (reg))//__raw_writeb((val), lcd_base + (reg))

static void lcd_write_inst(u8 cmd)
{
	
	//u8 __iomem* pLCD = (void __iomem *)(at91_lcd_base);
	
	
	//	*(pLCD) = cmd;
	at91_lcm_write(0x0 , cmd);
}

static u8 lcd_read_status(void)
{
	
	//u8 __iomem* pLCD = (void __iomem *)(at91_lcd_base);
	
	//return *(pLCD+0x04);
	return at91_lcm_read(0x4);
}



static void lcd_write_data(u8 data)
{
	//u8 __iomem* pLCD = (void __iomem *)(at91_lcd_base);
	
	//*(pLCD+0x1) = data;
	at91_lcm_write(0x1 , data);

}

static u8 lcd_read_data(void)
{
	//u8 __iomem* pLCD = (void __iomem *)(at91_lcd_base);
	
	// return *(pLCD+0x5);
	 return at91_lcm_read(0x5);
	
}

static void lcd_inst_set(int i)
{
	/* 8 bit interface */
	lcd_write_inst(0x30);
	udelay(1000);
	if(i == 1)
		lcd_write_inst(0x34); /* extended instruction */
	else
		lcd_write_inst(0x30);/* basic instruction */
	udelay(100);
}



static int lcd_ioctl(struct inode *inode, struct file *file,
		     unsigned int cmd, unsigned long arg)
{
	unsigned long address, a;

	switch (cmd) {
	case LCD_On:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x0c);
		break;
	
	case LCD_Off:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x08);
		break;
	case LCD_Back_On:
		pca9539_gpio_direction_output(0,0,0);  //40
		//at91_set_gpio_value(AT91_PIN_PC12, 0);
		break;

	case LCD_Back_Off:
		pca9539_gpio_direction_output(0,1,0);  //40
		//at91_set_gpio_value(AT91_PIN_PD24, 1);
	break;

	case LCD_Reset:
		udelay(150);
		lcd_write_inst(0x3F);
		udelay(150);
		lcd_write_inst(0x3F);
		udelay(150);
		lcd_write_inst(0x3F);
		udelay(150);
		lcd_write_inst(0x3F);
		udelay(150);
		lcd_write_inst(0x01);
		udelay(150);
		lcd_write_inst(0x06);
		break;

	case LCD_Clear:
		udelay(150);
		//BusyCheck();
		lcd_write_inst(0x01);
		break;

	case LCD_Cursor_Left:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x10);
		break;

	case LCD_Cursor_Right:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x14);
		break;

	case LCD_Cursor_Off:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x0c);
		break;

	case LCD_Cursor_On:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x0f);
		break;

	case LCD_Blink_Off:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x0e);
		break;

	case LCD_Get_Cursor_Pos:{
			
		}
	break;

	
	case LCD_Set_Cursor_Pos:{
			
			
		}
	break;
	case LCD_Get_Cursor:{
			

			break;
		}

	case LCD_Set_Cursor:{
			

			break;
		}


	case LCD_Disp_Left:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x18);
		break;

	case LCD_Disp_Right:
		udelay(150);
		BusyCheck();
		lcd_write_inst(0x1c);
		break;

	case LCD_Home:
		udelay(150);
		//BusyCheck();
		lcd_write_inst(0x02);
		break;

	case LCD_Write:{
			struct lcd_display display;
			unsigned int index;


			if (copy_from_user
			    (&display, (struct lcd_display *) arg,
			     sizeof(struct lcd_display)))
				return -EFAULT;

			udelay(150);
			BusyCheck();
			/* set DDRAM to 0 */
			lcd_write_inst(0x80);
			udelay(150);
			BusyCheck();

			for (index = 0; index < (display.size1); index++) {
				udelay(150);
				//BusyCheck();
				lcd_write_data(display.line1[index]);
				//BusyCheck();
			}

			udelay(150);
			//BusyCheck();
			lcd_write_inst(0x90);
			udelay(150);
			//BusyCheck();
			for (index = 0; index < (display.size2); index++) {
				udelay(150);
				//BusyCheck();
				lcd_write_data(display.line2[index]);
			}

			break;
		}

	case LCD_Read:{
			struct lcd_display display;

			BusyCheck();
			for (address = kDD_R00; address <= kDD_R01;
			     address++) {
				a = (address | kLCD_Addr);

				udelay(150);
				BusyCheck();
				lcd_write_inst(a);
				udelay(150);
				BusyCheck();
				display.line1[address] = lcd_read_data();
			}

			display.line1[0x27] = '\0';

			for (address = kDD_R10; address <= kDD_R11;
			     address++) {
				a = (address | kLCD_Addr);

				udelay(150);
				BusyCheck();
				lcd_write_inst(a);

				udelay(150);
				BusyCheck();
				display.line2[address - 0x40] =
				    lcd_read_data();
			}

			display.line2[0x27] = '\0';

			if (copy_to_user
			    ((struct lcd_display *) arg, &display,
			     sizeof(struct lcd_display)))
				return -EFAULT;
			break;
		}



	default:
		return -EINVAL;

	}

	return 0;

}

static int lcd_open(struct inode *inode, struct file *file)
{
	if (!lcd_present)
		return -ENXIO;
	else
		return 0;
}



/*
 *	The various file operations we support.
 */

static struct file_operations lcd_fops = {
	//.read = lcd_read,
	.ioctl = lcd_ioctl,
	.open = lcd_open,
};

static struct miscdevice lcd_dev = {
	MISC_LCD_MINOR,
	"lcd",
	&lcd_fops
};

static int lcd_init(void)
{
	int ret;
	unsigned long csa;

	pr_info("%s\n", LCD_DRIVER);
	ret = misc_register(&lcd_dev);
	if (ret) {
		printk(KERN_WARNING "Unable to register misc device.\n");
		return ret;
	}
	
	lcd_base = (unsigned long) ioremap(AT91_CHIPSELECT_0, SZ_16K); //WR
	if(!lcd_base) {
		printk("ioremap AT91 LCD failed\n");
		return 0;
	}
	
	at91_sys_write(AT91_SMC_SETUP(0),
		#if 1
		  AT91_SMC_NWESETUP_(63)
		| AT91_SMC_NCS_WRSETUP_(63)
		| AT91_SMC_NRDSETUP_(63)
		| AT91_SMC_NCS_RDSETUP_(63)
		#else
		  AT91_SMC_NWESETUP_(0)
		| AT91_SMC_NCS_WRSETUP_(2)
		| AT91_SMC_NRDSETUP_(0)
		| AT91_SMC_NCS_RDSETUP_(2)
		#endif
	);

	/* Pulse register */
	at91_sys_write(AT91_SMC_PULSE(0),
		#if 1
		  AT91_SMC_NWEPULSE_(126)
		| AT91_SMC_NCS_WRPULSE_(126)
                | AT91_SMC_NRDPULSE_(256 )
		| AT91_SMC_NCS_RDPULSE_(126)
		#else
		  AT91_SMC_NWEPULSE_(511)
		| AT91_SMC_NCS_WRPULSE_(511)
                | AT91_SMC_NRDPULSE_(511)
		| AT91_SMC_NCS_RDPULSE_(511)
		#endif
	);

	/* Cycle register */
	at91_sys_write(AT91_SMC_CYCLE(0),
		#if 1
		  AT91_SMC_NWECYCLE_(511)
		| AT91_SMC_NRDCYCLE_(511)
		#else
		 AT91_SMC_NWECYCLE_(7)
		| AT91_SMC_NRDCYCLE_(7)
		#endif
	);

	/* Mode register */
	at91_sys_write(AT91_SMC_MODE(0),
		   AT91_SMC_DBW_8 
		   //| AT91_SMC_EXNWMODE
		  | AT91_SMC_EXNWMODE_DISABLE
		 // | AT91_SMC_TDF_(0)
		//  | AT91_SMC_TDFMODE
	);

	
	return 0;		
	
}

static void __exit lcd_exit(void)
{
	iounmap(lcd_base);
	misc_deregister(&lcd_dev);
}



module_init(lcd_init);
module_exit(lcd_exit);

MODULE_AUTHOR("Alan Lu");
MODULE_LICENSE("GPL");
