/*
 * GPIO driver for Artila M502 based on Atmel AT91SAM9G20 (Thunder)
 *
 *  Copyright (C) 2010 Artila Electronics,Ltd
 *
 */

#include <linux/module.h>
#include <linux/fs.h>
#include <linux/miscdevice.h>
#include <asm/bitops.h> 
#include <asm/uaccess.h>
#include <linux/init.h>
#include <linux/spinlock.h>
#include <mach/hardware.h>
#include <mach/at91_pio.h>
#include <mach/gpio.h>
#include <linux/i2c.h>
#include <asm/pca9539.h>
#include <linux/interrupt.h>
#include <linux/irq.h>
#include <asm/delay.h>
#include <linux/spinlock.h>
#include <linux/poll.h>
#include <linux/proc_fs.h> 

#define GPIO_MINOR 		254

#define PIN  0
#define OUTP 1
#define INP  2
#define LOW  0
#define HIGH 1

/* IOCTL Commands */
#define GPIO_IOCTL_COUNT	0
#define GPIO_IOCTL_DIPSW	1
#define GPIO_IOCTL_GET		2
#define	GPIO_IOCTL_SET		3
#define	GPIO_IOCTL_CLEAR	4
#define GPIO_IOCTL_OUTPUT	5
#define GPIO_IOCTL_INPUT	6
#define GPIO_IOCTL_MODE		7
#define GPIO_IOCTL_INPUT_SET	8
#define GPIO_IOCTL_STATUS	0xa001

#define GPIO_INTERRUPTS_EN   0xb001
#define GPIO_INTERRUPTS_DS   0xb002
#define GPIO_INTERRUPTS_CR   0xb003
#define GPIO_INTERRUPTS_TS   0xb004
#define GPIO_INTERRUPTS_MASK 0xb005

//static DEFINE_SPINLOCK(gpio_lock);

int chips = 0;
int init_error = 0;


#ifdef CONFIG_MACH_M502

#define M504_GPIO_COUNT	24
#define M504_GPIO_SW	2
static int M502_GPIO[M504_GPIO_COUNT+M504_GPIO_SW] = {
	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,
	AT91_PIN_PC15,
	16,17,
#if defined(CONFIG_SND_AT91_SOC_SAM9G20_TLV320AIC23) || defined(CONFIG_SND_AT91_SOC_SAM9G20_TLV320AIC23_MODULE)
	18,//none pin
#else
	AT91_PIN_PC1,
#endif
	19,20,21,22,23,
	AT91_PIN_PB12,
	AT91_PIN_PB13,
};
static int M502_MODE[M504_GPIO_COUNT+M504_GPIO_SW];

#elif defined(CONFIG_MACH_M502_SOM)

#define M504_GPIO_COUNT	32
#define M504_GPIO_SW	2
static int M502_GPIO[M504_GPIO_COUNT+M504_GPIO_SW] = {
	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,
	AT91_PIN_PC15,
	16,17,
#if defined(CONFIG_SND_AT91_SOC_SAM9G20_TLV320AIC23) || defined(CONFIG_SND_AT91_SOC_SAM9G20_TLV320AIC23_MODULE)
	18,//none pin
#else
	AT91_PIN_PC1,
#endif
	19,20,21,22,23,24,25,26,27,28,29,30,31,
	AT91_PIN_PB12,
	AT91_PIN_PB13,
};
static int M502_MODE[M504_GPIO_COUNT+M504_GPIO_SW];

#elif defined(CONFIG_MACH_PAC4010)

static int M504_GPIO_COUNT = 0;
#define M504_GPIO_SW	0
static int gpio_pin0[16] = {
	0,1, 2, 3, 4, 5, 6, 7,
	8,9,10,11,12,13,14,16,
};
static int gpio_pin1[8] = {
	5,6,7,8,9,10,11,12
};

static int gpio_pin7[37] = {
	AT91_PIN_PB1,
	AT91_PIN_PB0,
	AT91_PIN_PB3,
	AT91_PIN_PB2,
	AT91_PIN_PA23,
	AT91_PIN_PC3,
	AT91_PIN_PB17,
	AT91_PIN_PA24,
	AT91_PIN_PB18,
	AT91_PIN_PB16,
	AT91_PIN_PB19,
	0,2, 1, 4, 3, 5, 6, 7,
	8,9,10,11,12,13,14,17,
	16,19,20,21,22,
	25,26,27,28,
};
static int M502_GPIO[36];

static int M502_MODE[36];
static int pac4010_id;

static struct proc_dir_entry *adc_entry;
#else

#define M504_GPIO_COUNT	21
#define M504_GPIO_SW	0
static int M502_GPIO[M504_GPIO_COUNT+M504_GPIO_SW] = {
	0,1,2,3,4,5,6,7,8,9,10,11,12,13,14, // 15
	21,
	16,17,
	22,
	19,20
};
static int M502_MODE[M504_GPIO_COUNT+M504_GPIO_SW];
#endif

#define AT91_GPIO_CNT 4
unsigned int AT91_GPIO[AT91_GPIO_CNT] = {AT91_PIN_PC15,AT91_PIN_PC1,AT91_PIN_PB12,AT91_PIN_PB13};
#define M504_TOTAL_GPIO	(M504_GPIO_COUNT+M504_GPIO_SW)
/* ......................................................................... */
#define PCA953X_INPUT          0
#define PCA953X_OUTPUT         1
#define PCA953X_INVERT         2
#define PCA953X_DIRECTION      3

static const struct i2c_device_id pca9539_id[] = {
	{ "pca9539", 16, },
	{ }
};
MODULE_DEVICE_TABLE(i2c, pca9539_id);

struct pca9539_chip {
	unsigned gpio_start;
	uint16_t reg_output;
	uint16_t reg_direction;

	struct i2c_client *client;
	
	uint16_t irq_mask;
	uint16_t irq_stat;
	uint16_t irq_trig_raise;
	uint16_t irq_trig_fall;
};

#define GPIO_POLL 1
#define M5_IRQ_EDGE_BOTH 	0x3
#define M5_IRQ_EDGE_FALLING 	0x1
#define M5_IRQ_EDGE_RISING 	0x2
struct gpio_set_irq {
        int trigger;
        int pin;
};

unsigned long IRQ_MASK;
unsigned long IRQ_TIMES[15];
spinlock_t		data_lock;
static struct fasync_struct *gpioirq_async_queue;
static DECLARE_WAIT_QUEUE_HEAD(gpio_wait);

struct pca9539_chip *save_chip[2];
spinlock_t		irq_lock;

static int pca9539_write_reg(struct pca9539_chip *chip, int reg, uint16_t val)
{
	int ret;
	unsigned long flags;
	spin_lock_irqsave(&irq_lock, flags);
	ret = i2c_smbus_write_word_data(chip->client, reg << 1, val);
	spin_unlock_irqrestore(&irq_lock, flags);
	if (ret < 0) {
		dev_err(&chip->client->dev, "failed writing register\n");
		return ret;
	}

	return 0;
}

static int pca9539_read_reg(struct pca9539_chip *chip, int reg, uint16_t *val)
{
	int ret;
	unsigned long flags;
	spin_lock_irqsave(&irq_lock, flags);
	ret = i2c_smbus_read_word_data(chip->client, reg << 1);
	spin_unlock_irqrestore(&irq_lock, flags);
	if (ret < 0) {
		dev_err(&chip->client->dev, "failed reading register\n");
		return ret;
	}

	*val = (uint16_t)ret;
	return 0;
}
static int pca9539_gpio_direction_input(unsigned off)
{
	struct pca9539_chip *chip;
	uint16_t reg_val;
	int ret;
	int save = off;
	
	if(M502_GPIO[save] >= 0 && M502_GPIO[save] <= 32) {
		
		off = M502_GPIO[save];
		
		chip = save_chip[off< 16 ? 0 : 1];
		if(off >= 16)
			off = off -16;
		reg_val = chip->reg_direction | (1u << off);
		ret = pca9539_write_reg(chip, PCA953X_DIRECTION, reg_val);
		if (ret)
			return ret;

		chip->reg_direction = reg_val;
	}else {
		at91_set_gpio_input(M502_GPIO[save], 0);
	}
	
	//M502_MODE[save] = 1;
	
	return 0;
}

int pca9539_gpio_direction_output(unsigned off, int val, int flag)
{
	struct pca9539_chip *chip;
	uint16_t reg_val;
	int ret;
	int save = off;
	
	if(flag == 1) {
		
		if(off < M504_TOTAL_GPIO)
		{
			if(M502_GPIO[save] >= 0 && M502_GPIO[save] <= 32) 
			{
				off = M502_GPIO[save];
			}
		}
		if(off >= 0 && off <= 32 && M502_GPIO[save] >= 0 && M502_GPIO[save] <= 32) {
			
			chip = save_chip[off < 16 ? 0 : 1];
			
			if(off >= 16)
				off = off -16;
			
			/* set output level */
			if (val)
				reg_val = chip->reg_output | (1u << off);
			else
				reg_val = chip->reg_output & ~(1u << off);
			
			
			ret = pca9539_write_reg(chip, PCA953X_OUTPUT, reg_val);
			if (ret) {
				return ret;
			}
			
			chip->reg_output = reg_val;
			
			/* then direction */
			reg_val = chip->reg_direction & ~(1u << off);
			ret = pca9539_write_reg(chip, PCA953X_DIRECTION, reg_val);
			if (ret)
				return ret;
			
			chip->reg_direction = reg_val;
			
		}else {
			
			if(flag) 
				at91_set_gpio_output(M502_GPIO[save], 0);
			
		}
		
	}else {
		chip = save_chip[off < 16 ? 0 : 1];
		
		if(off >= 16)
			off = off -16;
		
		/* set output level */
		if (val)
			reg_val = chip->reg_output | (1u << off);
		else
			reg_val = chip->reg_output & ~(1u << off);
		
		
		ret = pca9539_write_reg(chip, PCA953X_OUTPUT, reg_val);
		if (ret) {
			return ret;
		}
		
		chip->reg_output = reg_val;
		
		/* then direction */
		reg_val = chip->reg_direction & ~(1u << off);
		ret = pca9539_write_reg(chip, PCA953X_DIRECTION, reg_val);
		if (ret)
			return ret;
		
		chip->reg_direction = reg_val;
	}
	
	
	
	//M502_MODE[save] = 0;
	
	return 0;
}

EXPORT_SYMBOL(pca9539_gpio_direction_output);

static int pca9539_gpio_get_value(unsigned off)
{
	struct pca9539_chip *chip;
	uint16_t reg_val;
	int ret;
	int save = off;

	if(M502_GPIO[save] >= 0 && M502_GPIO[save] <= 32) {
		
		off = M502_GPIO[save];
		
		chip = save_chip[off < 16 ? 0 : 1];
		
		if(off >= 16)
			off = off -16;
		ret = pca9539_read_reg(chip, PCA953X_INPUT, &reg_val);
		if (ret < 0) {
			/* NOTE:  diagnostic already emitted; that's all we should
			* do unless gpio_*_value_cansleep() calls become different
			* from their nonsleeping siblings (and report faults).
			*/
			printk("Error\n");
			return 0;
		}
		return (reg_val & (1u << off)) ? 1 : 0;
	}
	
	return at91_get_gpio_value(M502_GPIO[off]);

	
}

void pca9539_gpio_set_value(unsigned off, int val , int flag)
{
	struct pca9539_chip *chip;
	uint16_t reg_val;
	int ret;
	int save = off;
	
	if(flag == 1) {
		if(off < M504_TOTAL_GPIO)
		{
			if(M502_GPIO[save] >= 0 && M502_GPIO[save] <= 32) {
				off = M502_GPIO[save];
			}
		}
		if((off >= 0 && off <= 32) && M502_GPIO[save] >= 0 && M502_GPIO[save] <= 32) 
		{
			chip = save_chip[off < 16 ? 0 : 1];
			
			if(off >= 16)
				off = off -16;
			if (val)
				reg_val = chip->reg_output | (1u << off);
			else
				reg_val = chip->reg_output & ~(1u << off);
			
			ret = pca9539_write_reg(chip, PCA953X_OUTPUT, reg_val);
			if (ret) {
				printk("Error\n");
				return;
			}
			
			chip->reg_output = reg_val;
			
		}else{
			
			if(flag)
				at91_set_gpio_output(M502_GPIO[save], val);
		}
		
	}else{
		chip = save_chip[off < 16 ? 0 : 1];
		
		if(off >= 16)
			off = off -16;
		if (val)
			reg_val = chip->reg_output | (1u << off);
		else
			reg_val = chip->reg_output & ~(1u << off);
		
		ret = pca9539_write_reg(chip, PCA953X_OUTPUT, reg_val);
		if (ret) {
			printk("Error\n");
			return;
		}
		
		chip->reg_output = reg_val;
	}
	
	
}

EXPORT_SYMBOL(pca9539_gpio_set_value);
#if 1 
//GPIO_IRQ
static uint16_t pca953x_irq_pending(void)
{
	struct pca9539_chip *chip;
	uint16_t cur_stat;
	uint16_t old_stat;
	uint16_t pending;
	uint16_t trigger;
	int ret;
	
	chip = save_chip[0];

	ret = pca9539_read_reg(chip, PCA953X_INPUT, &cur_stat);
	if (ret)
		return 0;

	/* Remove output pins from the equation */
	cur_stat &= chip->reg_direction;

	old_stat = chip->irq_stat;
	trigger = (cur_stat ^ old_stat) & chip->irq_mask;

	if (!trigger)
		return 0;

	chip->irq_stat = cur_stat;

	//pending = (old_stat ) | (cur_stat );
	pending = (old_stat & chip->irq_trig_fall) |
		  (cur_stat & chip->irq_trig_raise);
	pending &= trigger;

	return pending;
}

static irqreturn_t  irq_handler(int irq, void *dev_id) {

	uint16_t pending , i;
	unsigned long flags;
	
	pending = pca953x_irq_pending();
	if (!pending)
		return IRQ_HANDLED;
	
	//printk("irq_handler IRQ %x \n",pending);
	
	spin_lock_irqsave(&data_lock, flags);
	IRQ_MASK |= pending;
	for(i=0;i < 15 ; i++) {
		if( (1 << i) & pending)
			IRQ_TIMES[i]++;
	}
	kill_fasync(&gpioirq_async_queue, SIGIO, POLL_IN);
	spin_unlock_irqrestore(&data_lock, flags);
	
	return IRQ_HANDLED;
}
#endif

/*
 * GPIO device is opened, and GPIO starts running.
 */
static int m502_gpio_open(struct inode *inode, struct file *file)
{
	//spin_lock(&gpio_lock);
	//gpio_open_cnt++;
	//spin_unlock(&gpio_lock);
	return 0;
}

/*
 * Close the GPIO device.
 * If CONFIG_GPIO_NOWAYOUT is NOT defined then the GPIO is also
 *  disabled.
 */
static int m502_gpio_close(struct inode *inode, struct file *file)
{
	//spin_lock(&gpio_lock);
	//spin_unlock(&gpio_lock);
	return 0;
}

#if defined(CONFIG_MACH_PAC4010)
static int pac_id_read(char *resp_buff, char **start, off_t offset,
			      int count, int *eof, void *data, int flag)
{
	char *p = resp_buff;
	int len;
  
	p += sprintf(p, "%d\n", pac4010_id);
	len = (p - resp_buff) - offset;
	if (len < 0) 
		len = 0;
	*eof = (len <= count) ? 1 : 0;
	*start = resp_buff + offset;
	return len;
}
#endif

int irq_init;
/*
 * Handle commands from user-space.
 */
static int m502_gpio_ioctl(struct inode *inode, struct file *file,
		unsigned int cmd, unsigned long arg)
{
	struct pca9539_chip *chip;
	void __user *argp = (void __user *)arg;
	int i ;//= *((int __user *) argp);
	int ret , type , mask;
	struct gpio_set_irq set_irq;
	unsigned long flags;

	switch(cmd){
		case GPIO_IOCTL_COUNT:
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id == 0x7)
				*((int __user *) argp) = 0;
			else
#endif
				*((int __user *) argp) = M504_GPIO_COUNT; 
		return 0;
		case GPIO_IOCTL_DIPSW:
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id == 0x7)
				*((int __user *) argp) = 0;
			else
#endif
				*((int __user *) argp) = M504_GPIO_SW; 
		return 0;
		case GPIO_IOCTL_GET:
			i = *((int __user *) argp);
			if(i >= M504_TOTAL_GPIO)
				return -EINVAL;	
			if(M504_GPIO_COUNT > i)
				return pca9539_gpio_get_value(i);
			else {
				return at91_get_gpio_value(M502_GPIO[i]);
			}

		case GPIO_IOCTL_SET:
			i = *((int __user *) argp);
			if(i >= M504_GPIO_COUNT  )
				return -EINVAL;	

			if(M502_MODE[i] == 1)
				return -EINVAL;
			else
				pca9539_gpio_set_value(*((int __user *) argp),1,1);
			
		return 0;

		case GPIO_IOCTL_CLEAR:
			i = *((int __user *) argp);
			if(i >= M504_GPIO_COUNT  )
				return -EINVAL;	

			if(M502_MODE[i] == 1)
				return -EINVAL;
			else
				pca9539_gpio_set_value(*((int __user *) argp),0,1);
		return 0;
		
		case GPIO_IOCTL_OUTPUT:
#if defined(CONFIG_MACH_PAC4010)			
			if(pac4010_id != 0x7)
				return 1;
#endif			
			i = *((int __user *) argp);
			if(i >= M504_GPIO_COUNT )
				return -EINVAL;
			pca9539_gpio_direction_output(i,0,1);
			M502_MODE[i] = 0;
			return 0;
			
		case GPIO_IOCTL_INPUT:
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id != 0x7)
				return 1;
#endif
			
			i = *((int __user *) argp);
			//printk("%d\n",i);
			if(i >= M504_GPIO_COUNT)
				return -EINVAL;
			pca9539_gpio_direction_input(i);
			M502_MODE[i] = 1;
			return 0;
		case	GPIO_IOCTL_MODE:
			i = *((int __user *) argp);
			if(i >= M504_TOTAL_GPIO)
				return -EINVAL;
				
			if(M504_GPIO_COUNT > i)
				return M502_MODE[i];
			else
				return 1;
#if defined(CONFIG_MACH_PAC4010)
		case 99:
			return pac4010_id;
#endif
#if 1
				//GPIO_IRQ
		case GPIO_INTERRUPTS_EN:
			
			if (copy_from_user
				(&set_irq, (struct  gpio_set_irq*) arg,
				sizeof(struct gpio_set_irq)))
					return -EFAULT;
					
			if(irq_init == 0){	
				if(request_irq(AT91_PIN_PA28 , irq_handler ,
						0 ,//IRQF_SHARED, //|0x00002000
					"m502_gpio",NULL))
				{
					printk("M502 GPIO Driver init IRQ Fail.\n");
					return -EBUSY;//Ace
				}
				irq_init = 1;
			}
			
			type = set_irq.trigger;
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id == 0x1) {
				i = gpio_pin1[set_irq.pin];
			}else{
				i = set_irq.pin;
			}
#else
			i = set_irq.pin;
#endif
			if((i >= 15) || (M502_MODE[set_irq.pin] == 0))
				return -EINVAL;
			
			if (!(type & M5_IRQ_EDGE_BOTH)) {
				return -EINVAL;
			}

			chip = save_chip[0];
			
			if(chip->irq_mask == 0) {
				ret = pca9539_read_reg(chip, PCA953X_INPUT,
				       &chip->irq_stat);
				if(ret < 0) {
					printk("GPIO_INTERRUPTS_EN Error\n");
					return -EINVAL;
				}
				
				chip->irq_stat &= chip->reg_direction;
				spin_lock_irqsave(&data_lock, flags);
				IRQ_MASK = 0;
				spin_unlock_irqrestore(&data_lock, flags);
				wake_up_interruptible(&gpio_wait);
				
			}
			//printk("GPIO_INTERRUPTS_EN\n");
			
			
			mask = 1 << i;
			
			if (type & M5_IRQ_EDGE_FALLING)
				chip->irq_trig_fall |= mask;
			else
				chip->irq_trig_fall &= ~mask;

			if (type & M5_IRQ_EDGE_RISING)
				chip->irq_trig_raise |= mask;
			else
				chip->irq_trig_raise &= ~mask;
			
			spin_lock_irqsave(&data_lock, flags);	
			IRQ_TIMES[i] = 0;
			chip->irq_mask |= 1 << i;
			spin_unlock_irqrestore(&data_lock, flags);
			return 0;
			
		case GPIO_INTERRUPTS_DS:
			i = *((int __user *) argp);
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id == 0x1) {
				i = gpio_pin1[i];
			}
#endif
			if(i >= 15)
				return -EINVAL;
			
			chip = save_chip[0];
			
			spin_lock_irqsave(&data_lock, flags);
			mask = 1 << i;
			chip->irq_mask &= ~mask;
			IRQ_MASK &= ~mask;
			IRQ_TIMES[i] = 0;
			spin_unlock_irqrestore(&data_lock, flags);
			return 0;
			
		case GPIO_INTERRUPTS_CR:
			i = *((int __user *) argp);
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id == 0x1) {
				i = gpio_pin1[i];
			}
#endif
			if(i >= 15)
				return -EINVAL;
			
			spin_lock_irqsave(&data_lock, flags);	
			mask = 1 << i;
			IRQ_MASK &= ~mask;
			IRQ_TIMES[i] = 0;
			spin_unlock_irqrestore(&data_lock, flags);
			
			return 0;
			
		case GPIO_INTERRUPTS_TS:
			i = *((int __user *) argp);
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id == 0x1) {
				i = gpio_pin1[i];
			}
#endif
			if(i >= 15)
				return -EINVAL;
			return IRQ_TIMES[i];
			
		case GPIO_INTERRUPTS_MASK:
#if defined(CONFIG_MACH_PAC4010)
			if(pac4010_id == 0x1) {
				*((unsigned long __user *) argp) = IRQ_MASK >> 5;
			}
			*((unsigned long __user *) argp) = IRQ_MASK;
#else
			*((unsigned long __user *) argp) = IRQ_MASK;
#endif
			spin_lock_irqsave(&data_lock, flags);	
			IRQ_MASK = 0x0;
			spin_unlock_irqrestore(&data_lock, flags);
			return 0;
#endif
		/*case 	GPIO_IOCTL_INPUT_SET:
			i = *((int __user *) argp);
			return at91_set_gpio_input(M504_GPIO_MAP[i][0], 1); 
		*/
		/*case	0xa001:
			for (i = M504_TOTAL_GPIO ; i >= 0 ; i--) {
				status <<= 1;
				if(at91_get_gpio_value(M504_GPIO_MAP[i]))
					status++;
			}
			*((unsigned long __user *) argp) = status; 
			return 0;*/		
		default:return -ENOIOCTLCMD; 
	}
	return -ENOIOCTLCMD;
}


static int gpio_fasync(int fd, struct file *file, int on)
{
	return fasync_helper(fd, file, on, &gpioirq_async_queue);
}

/* ......................................................................... */

static struct file_operations m502gpio_fops =
{
	.owner		= THIS_MODULE,
	.ioctl		= m502_gpio_ioctl,
	.open		= m502_gpio_open,
	.release	= m502_gpio_close,
	.fasync		= gpio_fasync,
};

static struct miscdevice m502_gpio_miscdev =
{
	.minor		= GPIO_MINOR,
	.name		= "gpio",
	.fops		= &m502gpio_fops,	
};


static int __devinit pca9539_probe(struct i2c_client *client,
				   const struct i2c_device_id *id)
{
	
	int ret ,i;

	if (!i2c_check_functionality(client->adapter, I2C_FUNC_I2C
				     | I2C_FUNC_SMBUS_BYTE_DATA|
				     I2C_FUNC_SMBUS_WORD_DATA)) {
		ret = -ENODEV;
		return -EINVAL;
	}

	save_chip[chips] = kzalloc(sizeof(struct pca9539_chip), GFP_KERNEL);
	if (save_chip[chips] == NULL)
		return -ENOMEM;
	
	//if(chips == 0) 
	//	mutex_init(&irq_lock);
	
	save_chip[chips]->irq_mask = 0;
	save_chip[chips]->irq_stat = 0;
	save_chip[chips]->client = client;
	save_chip[chips]->gpio_start = id->driver_data;

	ret = pca9539_read_reg(save_chip[chips], PCA953X_OUTPUT, &save_chip[chips]->reg_output);
	if (ret)
		goto out_failed;

	ret = pca9539_read_reg(save_chip[chips], PCA953X_DIRECTION, &save_chip[chips]->reg_direction);
	if (ret)
		goto out_failed;
	
	ret = pca9539_write_reg(save_chip[chips], PCA953X_INVERT, 0);
	if (ret)
		goto out_failed;
	
	i2c_set_clientdata(client, save_chip[chips]);
	
	if(chips == 0) {
		chips = 1;
  
		//GPIO_IRQ
		at91_set_gpio_input(AT91_PIN_PA28, 0);
		irq_init = 0;
		
#if 0
		if(request_irq(AT91_PIN_PA28 , irq_handler ,
					 0 ,//IRQF_SHARED, //|0x00002000
				"m502_gpio",NULL))
		{
			printk("M502 GPIO Driver init IRQ Fail.\n");
			return -EBUSY;//Ace
		}

#endif	
		
		spin_lock_init(&(irq_lock));	
	}
	else {
		if(init_error == 0) {
			ret = misc_register(&m502_gpio_miscdev);
			if (ret)
				goto out_failed;
#if defined(CONFIG_MACH_PAC4010)
			memset(M502_GPIO,0,36);
			pac4010_id =  (at91_get_gpio_value(AT91_PIN_PB20) << 2 ) +
				      ( at91_get_gpio_value(AT91_PIN_PB21) << 1 ) +
				      ( at91_get_gpio_value(AT91_PIN_PC1) << 0 );
			printk("pac4010_id = 0x%x\n",pac4010_id)  ;   
			switch(pac4010_id) {
				case 0x0:
					M504_GPIO_COUNT = 16;
					for(i =0 ; i<M504_GPIO_COUNT ; i++)
						M502_GPIO[i] = gpio_pin0[i];
					
					for(ret = 0 ; ret < 8 ; ret++) {
							M502_MODE[ret] = 1;
							pca9539_gpio_direction_input(ret);
					}	
					for(ret = 8 ; ret < 16 ; ret++) {
							M502_MODE[ret] = 0;
							pca9539_gpio_direction_output(ret,0,1);
					}
					
					break;
				case 0x1:
					M504_GPIO_COUNT = 8;
					for(i =0 ; i<M504_GPIO_COUNT ; i++)
						M502_GPIO[i] = gpio_pin1[i];
					
					for(ret = 0 ; ret < 4 ; ret++) {
							M502_MODE[ret] = 1;
							pca9539_gpio_direction_input(ret);
					}	
					for(ret = 4 ; ret < 8 ; ret++) {
							M502_MODE[ret] = 0;
							pca9539_gpio_direction_output(ret,0,1);
					}
					pca9539_gpio_direction_output(0,0,0);
					pca9539_gpio_direction_output(1,0,0);
					break;
				case 0x7:
					M504_GPIO_COUNT = 37;
					for(i =0 ; i<M504_GPIO_COUNT ; i++)
						M502_GPIO[i] = gpio_pin7[i];
					
					break;
			}
			
			/*set uart to RS232*/
			for(ret = 23 ; ret < 25 ; ret++) {
				pca9539_gpio_direction_output(ret , 1 , 0);
				pca9539_gpio_set_value(ret , 1, 0);
			}
			struct proc_dir_entry *ent;
			//int i, j, ret;
	
			/* Set up the proc file system */
			adc_entry = proc_mkdir("pac", NULL);
			if (!adc_entry) {
				printk("Can't create /proc/adc\n");
				return  -ENOMEM;
			}
			
			adc_entry->owner=THIS_MODULE;
			
			
			ent = create_proc_entry("id", 0644, adc_entry);
			if (!ent) {
				printk("Error : Gpio Driver\n");
				ret = -ENOMEM;
				//goto fail;
				return  -ENOMEM;
			}
			ent->data = NULL;
			ent->read_proc = pac_id_read;
			ent->owner=THIS_MODULE;
			
#else	
			for(ret = 0 ; ret < M504_GPIO_COUNT ; ret++) {
					M502_MODE[ret] = 1;
					pca9539_gpio_direction_input(ret);
					//at91_set_gpio_input(M502_GPIO[ret], 0);
			}
			
			if(M504_GPIO_SW != 0) {
				for(ret = M504_GPIO_COUNT ; ret < M504_TOTAL_GPIO ; ret++)
						at91_set_gpio_input(M502_GPIO[ret], 1);
			}
				
			//printk("M502 GPIO Driver Ver 1.03 Loaded.\n");
			
			/*START UART SETTING*/
#if defined(CONFIG_MACH_M502)
			for(ret = 24 ; ret < 32 ; ret+=2) {
				pca9539_gpio_direction_output(ret , 1 , 0);
				pca9539_gpio_set_value(ret , 1, 0);
			}
			pca9539_gpio_direction_output(25 , 0 , 0);
			pca9539_gpio_set_value(25 , 0 , 0);
#elif defined(CONFIG_MACH_MATRIX518)
			for(ret = 23 ; ret < 32 ; ret++) {
				pca9539_gpio_direction_output(ret , 1 , 0);
				pca9539_gpio_set_value(ret , 1, 0);
			}
#elif defined(CONFIG_MACH_MATRIX522)
			for(ret = 23 ; ret < 25 ; ret++) {
				pca9539_gpio_direction_output(ret , 1 , 0);
				pca9539_gpio_set_value(ret , 1, 0);
			}
#endif
			/*END UART SETTING*/
#endif

			printk("M502 GPIO Driver Ver 1.04 Loaded.\n");	
		}
	}
	
	return 0;

out_failed:
	init_error = 1;
	if(chips == 0) {
		chips = 1;
	}
	kfree(save_chip[chips]);
	return ret;
}

static int pca9539_remove(struct i2c_client *client)
{
	struct pca9539_chip *chip = i2c_get_clientdata(client);
	misc_deregister(&m502_gpio_miscdev);
	kfree(chip);
	return 0;
}

static struct i2c_driver pca9539_driver = {
	.driver = {
		.name	= "pca9539",
	},
	.probe		= pca9539_probe,
	.remove		= pca9539_remove,
	.id_table	= pca9539_id,
};

static int __init m502_gpio_init(void)
{
	return i2c_add_driver(&pca9539_driver);
}

static void __exit m502_gpio_exit(void)
{
	i2c_del_driver(&pca9539_driver);
	
}

module_init(m502_gpio_init);
module_exit(m502_gpio_exit);

MODULE_LICENSE("GPL")
MODULE_AUTHOR("Ace Yang")
MODULE_DESCRIPTION("GPIO driver for M502 based on AT91SAM9G20")
